'use strict';
var postcss = require('postcss');
var objectAssign = require('object-assign');

var pxRegex = /"[^"]+"|'[^']+'|url\([^\)]+\)|(\d*\.?\d+)px/ig;
var defaults = {
    viewportWidth: 1080,
    unitPrecision: 5,
    minPixelValue: 1,
    viewportUnit: 'rem',
    uncoverSelectors: []
};

function createPxReplace(viewportSize, minPixelValue, unitPrecision, viewportUnit) {
    return function (m, $1) {
        if (!$1) return m;
        var pixels = parseFloat($1);
        if (pixels <= minPixelValue) return m;
        return (pixels / (viewportSize / 10)).toFixed(unitPrecision) + viewportUnit;
    };
}

function enableCover(blacklist, selector) {
    if (typeof selector !== 'string') return;
    return blacklist.some(function (regex) {
        if (typeof regex === 'string') return selector.indexOf(regex) !== -1;
        return selector.match(regex);
    });
}


module.exports = postcss.plugin('postcss-pxtrem', function (options) {
    var opts = objectAssign({}, defaults, options);
    var pxReplace = createPxReplace(opts.viewportWidth, opts.minPixelValue, opts.unitPrecision, opts.viewportUnit);
    return function (css) {
        css.walkDecls(function (decl) {
            if (decl.value.indexOf('px') === -1) return;
            if (enableCover(opts.uncoverSelectors, decl.parent.selector)) return;
            decl.value = decl.value.replace(pxRegex, pxReplace);
        });
    };
});